"""Airbrake-Python package attributes and metadata."""

__all__ = (
    '__title__',
    '__summary__',
    '__author__',
    '__email__',
    '__version__',
    '__keywords__',
    '__license__',
    '__url__',
    '__notifier__',
)

__title__ = 'airbrake'
__summary__ = 'Python SDK for airbrake.io'
__author__ = 'BK Box, Sam Stavinoha'
__email__ = 'samuel.stavinoha@rackspace.com'
__version__ = '1.3.3'
__keywords__ = ['airbrake', 'exceptions', 'airbrake.io']
__license__ = 'Apache License, Version 2.0'
__url__ = 'https://github.com/airbrake/airbrake-python'
__notifier__ = {
    'name': 'airbrake-python',
    'version': __version__,
    'url': __url__,
}
